@@ -4,6 +4,8 @@ import math, os, sys
 import ctypes.util
 from cffi.backend_ctypes import CTypesBackend
 from testing.udir import udir
+from testing.support import FdWriteCapture
+from .backend_tests import needs_dlopen_none
 
 try:
     from StringIO import StringIO
@@ -11,29 +13,6 @@ except ImportError:
     from io import StringIO
 
 
-class FdWriteCapture(object):
-    """xxx limited to capture at most 512 bytes of output, according
-    to the Posix manual."""
-
-    def __init__(self, capture_fd):
-        self.capture_fd = capture_fd
-
-    def __enter__(self):
-        self.read_fd, self.write_fd = os.pipe()
-        self.copy_fd = os.dup(self.capture_fd)
-        os.dup2(self.write_fd, self.capture_fd)
-        return self
-
-    def __exit__(self, *args):
-        os.dup2(self.copy_fd, self.capture_fd)
-        os.close(self.copy_fd)
-        os.close(self.write_fd)
-        self._value = os.read(self.read_fd, 512)
-        os.close(self.read_fd)
-
-    def getvalue(self):
-        return self._value
-
 lib_m = 'm'
 if sys.platform == 'win32':
     #there is a small chance this fails on Mingw via environ $CC
@@ -133,9 +112,10 @@ class TestFunction(object):
             int fputs(const char *, void *);
             void *stderr;
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         ffi.C.fputs   # fetch before capturing, for easier debugging
-        with FdWriteCapture(2) as fd:
+        with FdWriteCapture() as fd:
             ffi.C.fputs(b"hello\n", ffi.C.stderr)
             ffi.C.fputs(b"  world\n", ffi.C.stderr)
         res = fd.getvalue()
@@ -149,9 +129,10 @@ class TestFunction(object):
             int fputs(char *, void *);
             void *stderr;
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         ffi.C.fputs   # fetch before capturing, for easier debugging
-        with FdWriteCapture(2) as fd:
+        with FdWriteCapture() as fd:
             ffi.C.fputs(b"hello\n", ffi.C.stderr)
             ffi.C.fputs(b"  world\n", ffi.C.stderr)
         res = fd.getvalue()
@@ -165,8 +146,9 @@ class TestFunction(object):
            int fprintf(void *, const char *format, ...);
            void *stderr;
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
-        with FdWriteCapture(2) as fd:
+        with FdWriteCapture() as fd:
             ffi.C.fprintf(ffi.C.stderr, b"hello with no arguments\n")
             ffi.C.fprintf(ffi.C.stderr,
                           b"hello, %s!\n", ffi.new("char[]", b"world"))
@@ -191,6 +173,7 @@ class TestFunction(object):
         ffi.cdef("""
            int printf(const char *format, ...);
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         e = py.test.raises(TypeError, ffi.C.printf, b"hello %d\n", 42)
         assert str(e.value) == ("argument 2 passed in the variadic part "
@@ -201,6 +184,7 @@ class TestFunction(object):
         ffi.cdef("""
             int puts(const char *);
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         fptr = ffi.C.puts
         assert ffi.typeof(fptr) == ffi.typeof("int(*)(const char*)")
@@ -224,11 +208,12 @@ class TestFunction(object):
             int fputs(const char *, void *);
             void *stderr;
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         fptr = ffi.cast("int(*)(const char *txt, void *)", ffi.C.fputs)
         assert fptr == ffi.C.fputs
         assert repr(fptr).startswith("<cdata 'int(*)(char *, void *)' 0x")
-        with FdWriteCapture(2) as fd:
+        with FdWriteCapture() as fd:
             fptr(b"world\n", ffi.C.stderr)
         res = fd.getvalue()
         assert res == b'world\n'
@@ -257,6 +242,7 @@ class TestFunction(object):
         ffi.cdef("""
             int strlen(char[]);
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         p = ffi.new("char[]", b"hello")
         res = ffi.C.strlen(p)
@@ -269,6 +255,7 @@ class TestFunction(object):
         ffi.cdef("""
             void *stdout;
         """)
+        needs_dlopen_none()
         C = ffi.dlopen(None)
         pout = C.stdout
         C.stdout = ffi.NULL
@@ -281,6 +268,7 @@ class TestFunction(object):
         ffi.cdef("""
             char *strchr(const char *s, int c);
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         p = ffi.new("char[]", b"hello world!")
         q = ffi.C.strchr(p, ord('w'))
@@ -297,6 +285,7 @@ class TestFunction(object):
             struct in_addr { unsigned int s_addr; };
             char *inet_ntoa(struct in_addr in);
         """)
+        needs_dlopen_none()
         ffi.C = ffi.dlopen(None)
         ina = ffi.new("struct in_addr *", [0x04040404])
         a = ffi.C.inet_ntoa(ina[0])
@@ -318,6 +307,7 @@ class TestFunction(object):
         filename = str(udir.join('fputs_custom_FILE'))
         ffi = FFI(backend=self.Backend())
         ffi.cdef("int fputs(const char *, FILE *);")
+        needs_dlopen_none()
         C = ffi.dlopen(None)
         with open(filename, 'wb') as f:
             f.write(b'[')
@@ -333,6 +323,7 @@ class TestFunction(object):
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""enum foo_e { AA, BB, CC=5, DD };
                     typedef enum { EE=-5, FF } some_enum_t;""")
+        needs_dlopen_none()
         lib = ffi.dlopen(None)
         assert lib.AA == 0
         assert lib.BB == 1
@@ -344,6 +335,7 @@ class TestFunction(object):
     def test_void_star_accepts_string(self):
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""int strlen(const void *);""")
+        needs_dlopen_none()
         lib = ffi.dlopen(None)
         res = lib.strlen(b"hello")
         assert res == 5
@@ -353,6 +345,7 @@ class TestFunction(object):
             py.test.skip("not supported by the ctypes backend")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""int strlen(signed char *);""")
+        needs_dlopen_none()
         lib = ffi.dlopen(None)
         res = lib.strlen(b"hello")
         assert res == 5
@@ -362,6 +355,7 @@ class TestFunction(object):
             py.test.skip("not supported by the ctypes backend")
         ffi = FFI(backend=self.Backend())
         ffi.cdef("""int strlen(unsigned char *);""")
+        needs_dlopen_none()
         lib = ffi.dlopen(None)
         res = lib.strlen(b"hello")
         assert res == 5
@@ -486,10 +480,22 @@ class TestFunction(object):
         ffi = FFI(backend=self.Backend())
         ffi.cdef("double __stdcall sin(double x);")     # stdcall ignored
         m = ffi.dlopen(lib_m)
-        if (sys.platform == 'win32' and sys.maxsize < 2**32 and 
+        if (sys.platform == 'win32' and sys.maxsize < 2**32 and
                 self.Backend is not CTypesBackend):
             assert "double(__stdcall *)(double)" in str(ffi.typeof(m.sin))
         else:
             assert "double(*)(double)" in str(ffi.typeof(m.sin))
         x = m.sin(1.23)
         assert x == math.sin(1.23)
+
+    def test_dir_on_dlopen_lib(self):
+        ffi = FFI(backend=self.Backend())
+        ffi.cdef("""
+            typedef enum { MYE1, MYE2 } myenum_t;
+            double myfunc(double);
+            double myvar;
+            const double myconst;
+            #define MYFOO 42
+        """)
+        m = ffi.dlopen(lib_m)
+        assert dir(m) == ['MYE1', 'MYE2', 'MYFOO', 'myconst', 'myfunc', 'myvar']
